@@ -44,20 +44,21 @@ enum eKanjiCode gKanjiCode = kUtf8;
 
 
 char* gStatmentKindStrs[kCommand+1] = {
+    "var", "global", "export", 
+    "ary", "ary_add", "ary_erase", 
+    "hash", "hash_add", "hash_erase",
     "subshell", "msleep", "true",  "false", "[",  
     "index", 
     "rindex", 
     "length",
     "lc",  "uc", "chomp", "substr", "eval", "fg", 
-    "bg", "cpg", "jobs", 
+    "bg", "obs", 
     "rehash", 
-    "kanjicode", "linefield", "var", "global", "calc", 
-    "export", "print",  "puts",
+    "kanjicode", "linefield", "calc", 
+     "print",  "puts",
     "load",
     "while", "break", "exit", "if", "split", 
     "x", "join", "select", "lines", "rows",
-    "ary_new", "ary_add", "ary_erase", 
-    "hash_new", "hash_add", "hash_erase",
     "match", "scan", "erase", "sub",  
     "read",
     "cd", 
@@ -165,10 +166,6 @@ hash_obj* gRegexsIM;
     // 正規表現のキャッシュ(ignore case|multi line)
     // キーは正規表現の文字列
     // 内容はregex_t
-hash_obj* gFReadBuffers;
-    // 読み込みバッファ
-    // キーはファイルディスクリプタ番号の文字列
-    // 内容は読み込んだファイルの内容
 hash_obj* gFWriteBuffers;
     // 書き込みバッファ
     // キーはファイルディスクリプタ番号の文字列
@@ -284,63 +281,6 @@ void string_pomch(string_obj* str, enum eLineField lf)
     }
 }
 
-// closeするときにそのファイルディスクリプタに
-// gFReadBuffersのバッファがあるがあれば削除
-// gFWriteBUffersのバッファがあればバッファを書き込んで削除
-BOOL xclose(int fd) 
-{
-    char buf[32];
-    snprintf(buf, 32, "%d", fd);
-
-    string_obj* str = hash_item(gFReadBuffers, buf);
-    if(str) {
-        string_delete(str);
-        hash_erase(gFReadBuffers, buf);
-    }
-
-    close(fd);
-    return TRUE;
-}
-
-// 配列変数にまつわる変数(_size)を正しい値に更新
-void update_ary_env(char* key)
-{
-    vector_obj* v = hash_item(gArrays, key);
-
-    if(v) {
-        char name[128];
-
-        snprintf(name, 128, "%s_size", key);
-        char number[128];
-        snprintf(number, 128, "%d", vector_size(v));
-
-        string_obj* str = hash_item(gGlobals, name);
-        if(str) string_delete(str);
-        hash_put(gGlobals, name, STRING_NEW(number));
-    }
-    else {
-        char name[128];
-        snprintf(name, 128, "%s_size", key);
-        string_obj* str = hash_item(gGlobals, name);
-        if(str) {
-            string_delete(str);
-            hash_erase(gGlobals, name);
-        }
-    }
-}
-
-// 配列変数にまつわる変数を消す
-void clear_ary_env(char* key)
-{
-    char name[128];
-
-    snprintf(name, 128, "%s_size", key);
-
-    string_obj* str = hash_item(gGlobals, name);
-    if(str) string_delete(str);
-    hash_erase(gGlobals, name);
-}
-
 // エラーメッセージを設定
 void err_msg(char* msg, char* sname, int line)
 {
@@ -404,27 +344,6 @@ BOOL is_env_name_char(char c)
             || c == '_';
 }
 
-// グロブの区切り文字かどうか
-BOOL is_glob_terminated_char(char* p)
-{
-    return *p=='\n' || *p=='\r' || *p==' ' || *p=='\t' || *p=='>' 
-    || *p=='&' || *p =='$' || *p=='(' || *p==')'
-    || *p=='\\' || *p=='|' || *p=='{' || *p=='}' 
-    || *p==';'
-    || *p=='\'' || *p=='"' || *p=='<' || *p=='>' || *p==':' 
-    || *p=='!';
-}
-
-// 
-BOOL is_terminated_char(char* p)
-{
-    return *p=='\'' || *p=='"' || *p == '$' || *p == '%' || *p== '~' || *p=='*'
-    || *p=='?' || *p=='[' || *p=='<' || *p=='>' || *p==' ' || *p=='\t' 
-    || *p==':'
-    || *p=='{' || *p=='|' || *p=='&' || *p==';' || *p=='\n' || *p=='\r' 
-    || *p=='#' || *p==0
-    || *p=='(' || parse_is_redirect(p) || parse_is_redirect2(p);
-}
 
 // 文字列をクォートして返す
 void saphire_get_quoted_fname(char* fname, string_obj* quoted_fname)
@@ -446,63 +365,25 @@ void saphire_get_quoted_fname(char* fname, string_obj* quoted_fname)
         }
     }
 }
-
-
-void get_quoted_linefield(char* str, string_obj* out)
-{
-    char* p = str;
-    while(*p) {
-        if(*p == '\n') {
-            string_push_back(out, "\\n");
-            p++;
-        }
-        else if(*p == '\r') {
-            string_push_back(out, "\\r");
-            p++;
-        }
-        else {
-            string_push_back2(out, *p++);
-        }
-    }
-}
-
 /////////////////////////////////////////////////////////
 // ローカル変数、グローバル変数に参照、代入
 /////////////////////////////////////////////////////////
 void saphire_set_local_var(char* name, char* value)
 {
-    if(vector_size(gStackFrame) == 0) {
-        string_obj* var = hash_item(gGlobals, name);
+    hash_obj* top_stack 
+        = vector_item(gStackFrame, vector_size(gStackFrame)-1);
 
-        if(var) {
-            string_put(var, value);
-        }
-        else {
-            hash_put(gGlobals, name, STRING_NEW(value));
-        }
+    string_obj* var = hash_item(top_stack, name);
+
+    if(var) {
+        string_put(var, value);
     }
     else {
-        hash_obj* top_stack 
-            = vector_item(gStackFrame, vector_size(gStackFrame)-1);
-
-        string_obj* var = hash_item(top_stack, name);
-
-        if(var) {
-            string_put(var, value);
-        }
-        else {
-            hash_put(top_stack , name, STRING_NEW(value));
-        }
+        hash_put(top_stack , name, STRING_NEW(value));
     }
 }
 
 void saphire_set_global_var(char* name, char* value)
-{
-    saphire_delete_same_name_var(name);
-    hash_put(gGlobals, name, STRING_NEW(value));
-}
-
-void saphire_delete_same_name_var(char* name)
 {
     string_obj* global = hash_item(gGlobals, name);
 
@@ -510,37 +391,10 @@ void saphire_delete_same_name_var(char* name)
         string_delete(global);
         hash_erase(gGlobals, name);
     }
-
-    vector_obj* v = hash_item(gArrays, name);
-
-    if(v) {
-        int i;
-        for(i=0; i<vector_size(v); i++) {
-            string_delete(vector_item(v, i));
-        }
-        vector_delete(v);
-        hash_erase(gArrays, name);
-        update_ary_env(name);
-    }
-
-    hash_obj* h = hash_item(gHashs, name);
-
-    if(h) {
-        hash_it* it = hash_loop_begin(h);
-        while(it != NULL) {
-            string_delete(hash_loop_item(it));
-            it = hash_loop_next(it);
-        }
-        hash_delete(h);
-        hash_erase(gHashs, name);
-    }
-
-    char* env = getenv(name);
-    if(env) {
-        unsetenv(name);
-    }
+    hash_put(gGlobals, name, STRING_NEW(value));
 }
 
+
 char* saphire_get_global_var(char* name)
 {
     string_obj* global = hash_item(gGlobals, name);
@@ -565,74 +419,55 @@ vector_obj* saphire_get_array(char* name)
 
 void saphire_set_array(char* name, vector_obj* array)
 {
-    saphire_delete_same_name_var(name);
+    vector_obj* v = hash_item(gArrays, name);
+
+    if(v) {
+        int i;
+        for(i=0; i<vector_size(v); i++) {
+            string_delete(vector_item(v, i));
+        }
+        vector_delete(v);
+        hash_erase(gArrays, name);
+    }
+
     hash_put(gArrays, name, array);
-    update_ary_env(name);
 }
 
 char* saphire_get_local_var(char* name)
 {
-    if(vector_size(gStackFrame) == 0) {
-        string_obj* var = hash_item(gGlobals, name);
-        if(var) 
-            return string_c_str(var);
-        else 
-            return NULL;
+    hash_obj* top_stack 
+        = vector_item(gStackFrame, vector_size(gStackFrame)-1);
+
+    string_obj* var = hash_item(top_stack, name);
+
+    if(var) {
+        return string_c_str(var);
     }
     else {
-        hash_obj* top_stack 
-            = vector_item(gStackFrame, vector_size(gStackFrame)-1);
-
-        string_obj* var = hash_item(top_stack, name);
-
-        if(var) {
-            return string_c_str(var);
-        }
-        else {
-            return NULL;
-        }
+        return NULL;
     }
 }
 
 void saphire_delete_local_var(char* name)
 {
-    if(vector_size(gStackFrame) == 0) {
-        string_obj* var = hash_item(gGlobals, name);
-        if(var) {
-            string_delete(var);
-            hash_erase(gGlobals, name);
-        }
+    hash_obj* top_stack 
+        = vector_item(gStackFrame, vector_size(gStackFrame)-1);
 
-        char name_size[1024];
-        snprintf(name_size, 1024, "%s_size", name);
+    string_obj* var = hash_item(top_stack, name);
 
-        string_obj* var_size = hash_item(gGlobals, name_size);
-
-        if(var_size) {
-            string_delete(var_size);
-            hash_erase(gGlobals, name_size);
-        }
+    if(var) {
+        string_delete(var);
+        hash_erase(top_stack, name);
     }
-    else {
-        hash_obj* top_stack 
-            = vector_item(gStackFrame, vector_size(gStackFrame)-1);
 
-        string_obj* var = hash_item(top_stack, name);
+    char name_size[1024];
+    snprintf(name_size, 1024, "%s_size", name);
 
-        if(var) {
-            string_delete(var);
-            hash_erase(top_stack, name);
-        }
+    string_obj* var_size = hash_item(gGlobals, name_size);
 
-        char name_size[1024];
-        snprintf(name_size, 1024, "%s_size", name);
-
-        string_obj* var_size = hash_item(gGlobals, name_size);
-
-        if(var_size) {
-            string_delete(var_size);
-            hash_erase(gGlobals, name_size);
-        }
+    if(var_size) {
+        string_delete(var_size);
+        hash_erase(gGlobals, name_size);
     }
 }
 
@@ -2889,7 +2724,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "2.0.2", 1);
+    setenv("SAPHIRE_VERSION", "3.0.0", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2912,13 +2747,12 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gRegexsI = HASH_NEW(100);
     gRegexsM = HASH_NEW(100);
     gRegexsIM = HASH_NEW(100);
-    gFReadBuffers = HASH_NEW(100);
     gFWriteBuffers = HASH_NEW(512);
     
     gStrayCProgs = VECTOR_NEW(10);
 
     gStackFrame = VECTOR_NEW(5);
-    //vector_add(gStackFrame, HASH_NEW(30));
+    vector_add(gStackFrame, HASH_NEW(30));
 
     gInnerCommands = HASH_NEW(50);
     gKitutukiExit = -1;
@@ -2927,6 +2761,9 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gDirStack = VECTOR_NEW(10);
     gErrMsg = STRING_NEW("");
 
+    gStackTraceFName = VECTOR_NEW(30);
+    gStackTraceLineNum = VECTOR_NEW(30);
+
 
 //    saphire_rehash();
 
@@ -2957,9 +2794,6 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     else 
         xstrncpy(gTmpDir, "/tmp", PATH_MAX);
 
-    gStackTraceFName = VECTOR_NEW(30);
-    gStackTraceLineNum = VECTOR_NEW(30);
-
     tmpfile_all_sweep();
     gSaphireVarCompletions = VECTOR_NEW(100);
 }
@@ -3092,13 +2926,6 @@ void saphire_final()
     }
     hash_delete(gRegexsIM);
     
-    it = hash_loop_begin(gFReadBuffers);
-    while(it != NULL) {
-        string_delete(hash_loop_item(it));
-        it = hash_loop_next(it);
-    }
-    hash_delete(gFReadBuffers);
-    
     it = hash_loop_begin(gFWriteBuffers);
     while(it != NULL) {
         string_delete(hash_loop_item(it));
